﻿using System;
using System.ComponentModel;

namespace UMK.WzorceOperacyjne
{
    //https://sourcemaking.com/design_patterns/observer/cpp/1

    interface IObserwator
    {
        void Aktualizuj(int dzielna);
    }

    class Dzielenie : IObserwator
    {
        private int wartość = 0;
        private int dzielnik;

        public Dzielenie(int dzielnik)
        {
            this.dzielnik = dzielnik;
        }

        public void Aktualizuj(int dzielna)
        {
            wartość = dzielna / dzielnik;
            Console.WriteLine("{0} / {1} = {2}", dzielna, dzielnik, wartość);
        }
    }

    class Modulo : IObserwator
    {
        private int wartość = 0;
        private int dzielnik;

        public Modulo(int dzielnik)
        {
            this.dzielnik = dzielnik;
        }

        public void Aktualizuj(int dzielna)
        {
            wartość = dzielna % dzielnik;
            Console.WriteLine("{0} % {1} = {2}", dzielna, dzielnik, wartość);
        }
    }

    class Podmiot //obserwabla
    {
        private int wartość;
        //private Dzielenie dzielenie; //obiekty, których stan zależy od zmieniającego się pola wartość
        //private Modulo modulo;
        private List<IObserwator> obserwatorzy = new List<IObserwator>();

        private void powiadom()
        {
            //dzielenie.Aktualizuj(wartość);
            //modulo.Aktualizuj(wartość);
            foreach (IObserwator obserwator in obserwatorzy)
                obserwator.Aktualizuj(wartość);
        }

        public void PodłączObserwatora(IObserwator obserwator) 
        { 
            obserwatorzy.Add(obserwator);
        }

        public void OdłączObserwatora(IObserwator obserwator)
        {
            obserwatorzy.Remove(obserwator);
        }

        public Podmiot(params IObserwator[] obserwatorzy)
        {
            wartość = 0;
            //dzielenie = new Dzielenie(3);
            //modulo = new Modulo(3);
            this.obserwatorzy.AddRange(obserwatorzy);
        }

        public void ZmieńWartość(int nowaWartość)
        {
            wartość = nowaWartość;
            powiadom();
            onZmienionaWartość(nowaWartość);
        }

        #region Zdarzenie
        public class ZmienionaWartośćEventArgs : EventArgs
        {
            public int NowaWartość;
        }

        //public delegate TZmienionaWartość(object sender, ZmienionaWartośćEventArgs e);

        public event EventHandler<ZmienionaWartośćEventArgs> ZmienionaWartość;

        private void onZmienionaWartość(int nowaWartość)
        {
            if (ZmienionaWartość != null) ZmienionaWartość(
                this,
                new ZmienionaWartośćEventArgs() { NowaWartość = nowaWartość });
        }
        #endregion
}
}